/************************************************************************/
/*																			*/
/*	Motor_Shield.cpp	This library supports the chipKIT motor shield,     */
/*                  	in particular, providing an easy means of using     */
/*						the I/O extender functionality                   	*/
/*																		*/
/************************************************************************/
/*	Author: 	James Irwin 											*/
/*	Copyright 2013, Digilent Inc.										*/
/************************************************************************/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/************************************************************************/
/*  Module Description: 												*/
/*																		*/
/*	This the Static Motor Shield function file             				*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	 2/5/2013(JIrwin): Created											*/
/*																		*/
/************************************************************************/

#include "MotorShield.h"

/*****************************************************************************
  Function:
    void initialize()
    
  Description:
	Initializes the I/O extender by configuring the GPIO as inputs and outputs

  Parameters:
    NONE
  
  Returns:
	NONE

  Remarks:

 ***************************************************************************/
void MotorShield::begin(void)
{	
	Wire.begin();	//joint the I2C bus as master
	
	Wire.beginTransmission(DEVADDR); // transmit to motor shield
	Wire.send(IODIR); //select the data direction register
	Wire.send(0xF0); //configure high half of I/O as inputs, and the low half as outputs
	Wire.endTransmission();    // stop transmitting

}

/*****************************************************************************
  Function:
    void writeLEDs()
    
  Description:
	writes a 4-bit value to the LEDs

  Parameters:
    byte, value to display to the LEDs
  
  Returns:
	NONE

  Remarks:

 ***************************************************************************/
void MotorShield::writeLEDs(byte value)
{	
	value = value & LEDMASK; //mask the value to only the bits we want (unnecessary, but good habit)
	
	reorder_LEDs(&value);	//reorder the leds to match silkscreen labels on the board
	
	Wire.beginTransmission(DEVADDR); // transmit to motor shield
	Wire.send(GPIO);        // select the GPIO register
	Wire.send(value);              // sets the value 
	Wire.endTransmission();    // stop transmitting  

}

/*****************************************************************************
  Function:
    void readInputs()
    
  Description:
	reads the status of the 2 buttons and 2 jumpers, expressed as a 4-bit value

  Parameters:
    NONE
  
  Returns:
	byte, lower 4 bits tell current state of the buttons

  Remarks:

 ***************************************************************************/
byte MotorShield::readInputs(void)
{	
	byte temp=0x00;
  
	Wire.beginTransmission(DEVADDR); // transmit to motor shield
	Wire.send(GPIO);				 // select the GPIC register
	Wire.endTransmission();			 // end transmission (not really ending, but neccessary step for reading)
	Wire.requestFrom(DEVADDR, 1);	 // tells the shield to send 1 byte of information. It will read from the register just selected 
	

	temp = Wire.receive(); //read the received information
	
	temp = temp & BTNMASK; 	//mask the value to only the btn bits (state of LEDs is also in the register, but we don't care about them)
	temp = temp >> 4;		//values are currently in high half of byte, move to lower half to make working with the values easier
	
	return temp;
}


/*****************************************************************************
  Function:
    void readButtons()
    
  Description:
	reads the status of the 2 pushbuttons

  Parameters:
    bool* , pointer to two flags that indicate button status (1 for pressed, 0 else)
  
  Returns:
	NONE

  Remarks:

 ***************************************************************************/
void MotorShield::readButtons(bool* btn1, bool* btn2)
{	
	byte temp = 0x00;
  
	temp = readInputs();
	
	temp = temp << 4; //value was shifted by readInputs(), shift back so masks work on proper bits.

	if (btn1 != NULL)
		*btn1 = (temp & BTN1MASK);
	
	if (btn2 != NULL)
		*btn2 = (temp & BTN2MASK);	

}


/*****************************************************************************
  Function:
    void reorder_LEDs()
    
  Description:
	re-arranges the bits such that bit 1 (lsb) corresponds to LED1 on the board,
	bit 2 to LED2, etc.

  Parameters:
    byte* value, pointer to byte to be rearranged
  
  Returns:
	NONE

  Remarks:
	This function is used in writeLEDs(), it is technically unneccessary and can
	be commented out

 ***************************************************************************/
void MotorShield::reorder_LEDs(byte* value)
{	
	byte temp = 0x00;
	
	if(*value&0x01)
		temp = temp | 0x08;
	if(*value&0x02)
		temp = temp | 0x04;
	if(*value&0x04)
		temp = temp | 0x02;
	if(*value&0x08)
		temp = temp | 0x01;	
		
	*value = temp;
}

